@@ -24,7 +24,10 @@
 #include <libglom/connectionpool_backends/postgres.h>
 #include <libglom/spawn_with_feedback.h>
 #include <libglom/utils.h>
-#include <giomm.h>
+#include <libgdamm/config.h>
+#include <giomm/file.h>
+#include <glibmm/convert.h>
+#include <glibmm/miscutils.h>
 #include <glib/gstdio.h> /* For g_rename(). TODO: Wrap this in glibmm? */
 #include <glibmm/i18n.h>
 
@@ -119,7 +122,7 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
 
   if(data_model && data_model->get_n_rows() && data_model->get_n_columns())
   {
-    Gnome::Gda::Value value = data_model->get_value_at(0, 0);
+    const Gnome::Gda::Value value = data_model->get_value_at(0, 0);
     if(value.get_value_type() == G_TYPE_STRING)
     {
       const Glib::ustring version_text = value.get_string();
@@ -397,7 +400,16 @@ bool Postgres::check_postgres_gda_client_is_available()
     {
       //See http://library.gnome.org/devel/libgda/unstable/libgda-40-Configuration.html#gda-config-list-providers
       //about the columns of this DataModel:
-      const Gnome::Gda::Value name = iter->get_value_at(0);
+      Gnome::Gda::Value name;
+      try
+      {
+        name = iter->get_value_at(0);
+      }
+      catch(const Glib::Error& ex)
+      {
+        std::cerr << G_STRFUNC << ": exception: " << ex.what() << std::endl;
+      }
+
       if(name.get_value_type() != G_TYPE_STRING)
         continue;
 
@@ -767,8 +779,8 @@ bool Postgres::create_directory_filepath(const std::string& filepath)
   const int mkdir_succeeded = g_mkdir_with_parents(filepath.c_str(), 0770);
   if(mkdir_succeeded == -1)
   {
-    std::cerr << G_STRFUNC << "Error from g_mkdir_with_parents() while trying to create directory: " << filepath << std::endl;
-    perror("Error from g_mkdir_with_parents");
+    std::cerr << G_STRFUNC << ": Error from g_mkdir_with_parents() while trying to create directory: " << filepath << std::endl;
+    perror("  perror(): Error from g_mkdir_with_parents()");
 
     return false;
   }
@@ -854,7 +866,7 @@ bool Postgres::create_text_file(const std::string& file_uri, const std::string&
     return false;
 
 
-  gsize bytes_written = 0;
+  gssize bytes_written = 0;
   const std::string::size_type contents_size = contents.size();
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
@@ -877,7 +889,7 @@ bool Postgres::create_text_file(const std::string& file_uri, const std::string&
     return false; //print_error(ex, output_uri_string);
   }
 
-  if(bytes_written != contents_size)
+  if(bytes_written != (gssize)contents_size)
   {
     std::cerr << "ConnectionPool::create_text_file(): not all bytes written when writing to file." << std::endl
       << "  file uri:" << file_uri << std::endl;
@@ -887,6 +899,28 @@ bool Postgres::create_text_file(const std::string& file_uri, const std::string&
   return true; //Success.
 }
 
+Field::sql_format Postgres::get_sql_format() const
+{
+  return Field::SQL_FORMAT_POSTGRES;
+}
+
+bool Postgres::supports_remote_access() const
+{
+  return true;
+}
+
+Gnome::Gda::SqlOperatorType Postgres::get_string_find_operator() const
+{
+  // ILIKE is a PostgreSQL extension for locale-dependent case-insensitive matches.
+  //See http://developer.postgresql.org/pgdocs/postgres/functions-matching.html
+  return Gnome::Gda::SQL_OPERATOR_TYPE_ILIKE;
+}
+
+const char* Postgres::get_public_schema_name() const
+{
+  return "public";
+}
+
 } //namespace ConnectionPoolBackends
 
 } //namespace Glom
