@@ -23,7 +23,6 @@
 #include "entry.h"
 #include "checkbutton.h"
 #include "label.h"
-#include <glom/mode_data/datawidget/comboentry.h>
 #include <glom/mode_data/datawidget/combo.h>
 #include <glom/mode_data/datawidget/combo_as_radio_buttons.h>
 #include <glom/mode_data/datawidget/textview.h>
@@ -36,10 +35,8 @@
 #include <glom/mode_design/layout/layout_item_dialogs/dialog_field_layout.h>
 #include <glom/utils_ui.h>
 #include <glom/glade_utils.h>
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/button.h>
-#endif //GLOM_ENABLE_MAEMO
+#include <gtkmm/stock.h>
+#include <libglom/db_utils.h>
 
 #include <glibmm/i18n.h>
 
@@ -59,7 +56,7 @@ static DataWidgetChildren::ComboChoices* create_combo_widget_for_field(const sha
       result = Gtk::manage(new DataWidgetChildren::ComboGlom());
   }
   else
-    result = Gtk::manage(new DataWidgetChildren::ComboEntry());
+    result = Gtk::manage(new DataWidgetChildren::ComboGlom(true /* has_entry */));
 
   return result;
 }
@@ -104,7 +101,6 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     m_label.set_alignment(0);
     m_label.show();
   }
-  // Use hildon widgets for date and time on maemo
   else
   {
     //The GNOME HIG says that labels should have ":" at the end:
@@ -120,22 +116,17 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 
       if(field->get_formatting_used().get_has_custom_choices())
       {
-        //set_choices() needs this, for the numeric layout:
+        //set_choices_fixed() needs this, for the numeric layout:
         combo->set_layout_item( get_layout_item(), table_name);
 
-        combo->set_choices( field->get_formatting_used().get_choices_custom() );
+        combo->set_choices_fixed( field->get_formatting_used().get_choices_custom() );
       }
       else if(field->get_formatting_used().get_has_related_choices())
       {
         combo = create_combo_widget_for_field(field);
         combo->set_layout_item( get_layout_item(), table_name);
 
-        sharedptr<const Relationship> choice_relationship;
-        Glib::ustring choice_field, choice_second;
-        bool choice_show_all = false;
-        field->get_formatting_used().get_choices_related(choice_relationship, choice_field, choice_second, choice_show_all);
-
-        combo->set_choices_related(document, choice_relationship, choice_field, choice_second, choice_show_all);
+        combo->set_choices_related(document, field, Gnome::Gda::Value() /* TODO: Doesn't make sense */);
       }
       else
       {
@@ -186,29 +177,21 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 
     bool child_added = false; //Don't use an extra container unless necessary.
 
-    //Check whether the field controls a relationship,
-    //meaning it identifies a record in another table.
-    const bool field_used_in_relationship_to_one = document->get_field_used_in_relationship_to_one(table_name, field);
-    //std::cout << "DEBUG: table_name=" << table_name << ", table_used=" << field->get_table_used(table_name) << ", field=" << field->get_name() << ", field_used_in_relationship_to_one=" << field_used_in_relationship_to_one << std::endl;
+    //Update the field details from the document:
+    field->set_full_field_details(
+      document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
 
-    //Check whether the field identifies a record in another table
-    //just because it is a primary key in that table:
-    bool field_is_related_primary_key = false;
-    if(document)
-      field->set_full_field_details( document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
-    sharedptr<const Field> field_info = field->get_full_field_details();
-    field_is_related_primary_key =
-      field->get_has_relationship_name() &&
-      field_info && field_info->get_primary_key();
-    //std::cout <<   "DEBUG: field->get_has_relationship_name()=" << field->get_has_relationship_name() << ", field_info->get_primary_key()=" <<  field_info->get_primary_key() << ", field_is_related_primary_key=" << field_is_related_primary_key << std::endl;
+    sharedptr<Relationship> field_used_in_relationship_to_one;
+    const bool add_open_button = 
+       DbUtils::layout_field_should_have_navigation(table_name, field, document, 
+         field_used_in_relationship_to_one);
 
+    Gtk::Box* hbox_parent = 0; //Only used if there are extra widgets.
 
-    Gtk::HBox* hbox_parent = 0; //Only used if there are extra widgets.
-
-    const bool with_extra_widgets = field_used_in_relationship_to_one || field_is_related_primary_key || (glom_type == Field::TYPE_DATE);
+    const bool with_extra_widgets = field_used_in_relationship_to_one || add_open_button || (glom_type == Field::TYPE_DATE);
     if(with_extra_widgets)
     {
-      hbox_parent = Gtk::manage( new Gtk::HBox() ); //We put the child (and any extra stuff) in this:
+      hbox_parent = Gtk::manage( new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL) ); //We put the child (and any extra stuff) in this:
       hbox_parent->set_spacing(Utils::DEFAULT_SPACING_SMALL);
 
       hbox_parent->pack_start(*m_child);
@@ -221,42 +204,29 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     if(glom_type == Field::TYPE_DATE)
     {
       //Let the user choose a date from a calendar dialog:
-      #ifndef GLOM_ENABLE_MAEMO
       Gtk::Button* button_date = Gtk::manage(new Gtk::Button(_("..."))); //TODO: A better label/icon for "Choose Date".
-      #else
-      Gtk::Button* button_date = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("..."), ""));
-      #endif
       button_date->set_tooltip_text(_("Choose a date from an on-screen calendar."));
       button_date->show();
       hbox_parent->pack_start(*button_date, Gtk::PACK_SHRINK);
       button_date->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_choose_date));
     }
 
-    if((field_used_in_relationship_to_one || field_is_related_primary_key) && hbox_parent)
+    if(hbox_parent && add_open_button)
     {
       //Add a button for related record navigation:
-      #ifndef GLOM_ENABLE_MAEMO
       m_button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
-      #else
-      m_button_go_to_details = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Open"), ""));
-      #endif
       m_button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
-      hbox_parent->pack_start(*m_button_go_to_details);
+      hbox_parent->pack_start(*m_button_go_to_details, Gtk::PACK_SHRINK);
       m_button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
 
-      //Add a button to make it easier to choose an ID for this field.
+      //Add an additional button to make it easier to choose an ID for this field.
       //Don't add this for simple related primary key fields, because they
       //can generally not be edited via another table's layout.
       if(field_used_in_relationship_to_one)
       {
-        #ifndef GLOM_ENABLE_MAEMO
         Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
-        #else
-        Gtk::Button* button_select = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT,
-           Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Find"), ""));
-        #endif
         button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
-        hbox_parent->pack_start(*button_select);
+        hbox_parent->pack_start(*button_select, Gtk::PACK_SHRINK);
         button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
       }
     }
@@ -270,7 +240,7 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 #endif // GLOM_ENABLE_CLIENT_ONLY
 
   set_events(Gdk::BUTTON_PRESS_MASK);
-  signal_style_changed().connect(sigc::mem_fun(*this, &DataWidget::on_self_style_changed));
+  //TODO: signal_style_changed().connect(sigc::mem_fun(*this, &DataWidget::on_self_style_changed));
 }
 
 DataWidget::~DataWidget()
@@ -296,7 +266,7 @@ void DataWidget::set_value(const Gnome::Gda::Value& value)
   if(generic_field_widget)
   {
     //if(generic_field_widget->get_layout_item())
-    //  std::cout << "DataWidget::set_value(): generic_field_widget->get_layout_item()->get_name()=" << generic_field_widget->get_layout_item()->get_name() << std::endl;
+    //  std::cout << "debug: " << G_STRFUNC << ": generic_field_widget->get_layout_item()->get_name()=" << generic_field_widget->get_layout_item()->get_name() << std::endl;
 
     generic_field_widget->set_value(value);
   }
@@ -368,7 +338,6 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
     int height = -1; //auto.
     if((glom_type == Field::TYPE_TEXT) && (field->get_formatting_used().get_text_format_multiline()))
     {
-      #ifndef GLOM_ENABLE_MAEMO
       int example_width = 0;
       int example_height = 0;
       Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout("example"); //TODO: Use different text, according to the current locale, or allow the user to choose an example?
@@ -376,13 +345,6 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
 
       if(example_height > 0)
         height = example_height * field->get_formatting_used().get_text_format_multiline_height_lines();
-      #else
-      //On Maemo, TextView widgets expand automatically.
-      //TODO: Expansion only happens if both are -1, and vertical expansion never happens.
-      //See bug https://bugs.maemo.org/show_bug.cgi?id=5515
-      width = -1;
-      height = -1;
-      #endif //GLOM_ENABLE_MAEMO
     }
 
     m_child->set_size_request(width, height);
@@ -450,7 +412,7 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
     if(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
     {
       GdkModifierType mods;
-      gdk_window_get_pointer( gtk_widget_get_window (Gtk::Widget::gobj()), 0, 0, &mods );
+      gdk_window_get_device_position( gtk_widget_get_window (Gtk::Widget::gobj()), event->device, 0, 0, &mods );
       if(mods & GDK_BUTTON3_MASK)
       {
         //Give user choices of actions on this item:
@@ -506,6 +468,8 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const
 
   Dialog_FieldLayout* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return result;
 
   add_view(dialog); //Give it access to the document.
   dialog->set_field(start_field, m_table_name);
